/**
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <system_info.h>

#include "mv_private.h"
#include "mv_barcode_detect.h"

#ifdef MEDIA_VISION_BARCODE_DETECTOR_LICENSE_PORT

/* Include headers of licensed barcode detect module here. */
#include "mv_barcode_detect_lic.h"

#else

/* Include headers of open barcode detect module here. */
#include "mv_barcode_detect_open.h"

#endif /* MEDIA_VISION_BARCODE_DETECTOR_LICENSE_PORT */

/**
 * @file  mv_barcode_detect.c
 * @brief This file contains the porting layer for Media Vision barcode module.
 */

int mv_barcode_detect(
		mv_source_h source,
		mv_engine_config_h engine_cfg,
		mv_rectangle_s roi,
		mv_barcode_detected_cb detect_cb,
		void *user_data)
{
	MEDIA_VISION_SUPPORT_CHECK(__mv_barcode_detect_check_system_info_feature_supported());
	MEDIA_VISION_INSTANCE_CHECK(source);
	MEDIA_VISION_NULL_ARG_CHECK(detect_cb);

	MEDIA_VISION_FUNCTION_ENTER();

	int ret = MEDIA_VISION_ERROR_NONE;
	unsigned int src_w = 0;
	unsigned int src_h = 0;

	ret = mv_source_get_width(source, &src_w);
	if (ret != MEDIA_VISION_ERROR_NONE) {
		LOGE("mv_source_get_width fail");
		return ret;
	}

	ret = mv_source_get_height(source, &src_h);
	if (ret != MEDIA_VISION_ERROR_NONE) {
		LOGE("mv_source_get_height fail");
		return ret;
	}

	if (roi.point.x < 0 || roi.point.y < 0 ||
		(roi.point.x + roi.width) > src_w ||
		(roi.point.y + roi.height) > src_h) {
		LOGE("roi is out of area on source");
		return MEDIA_VISION_ERROR_INVALID_PARAMETER;
	}

#ifdef MEDIA_VISION_BARCODE_DETECTOR_LICENSE_PORT

	/* Use licensed barcode detect functionality here. */
	ret = mv_barcode_detect_lic(
					source, engine_cfg, roi, detect_cb, user_data);

#else

	/* Use open barcode detect functionality here. */
	ret = mv_barcode_detect_open(
					source, engine_cfg, roi, detect_cb, user_data);

#endif /* MEDIA_VISION_BARCODE_DETECTOR_LICENSE_PORT */

	MEDIA_VISION_FUNCTION_LEAVE();
	return ret;
}
