/**
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Tracking/FeatureSubstitutionTracker.h"

#include "Recognition/ImageRecognizer.h"

#include <new>

namespace MediaVision {
namespace Image {

FeatureSubstitutionTracker::FeatureSubstitutionTracker(
		const FeaturesExtractingParams& featuresExtractingParams,
		const RecognitionParams& recognitionParams,
		float expectedOffset,
		float sceneScalingFactor,
		float objectScalingFactor) :
				__isInit(false),
				__target(),
				__location(),
				__featureExtractingParams(featuresExtractingParams),
				__recogParams(recognitionParams),
				__expectedOffset(expectedOffset),
				__sceneScalingFactor(sceneScalingFactor),
				__objectScalingFactor(objectScalingFactor)
{
	; /* NULL */
}

bool FeatureSubstitutionTracker::track(
		const cv::Mat& frame,
		std::vector<cv::Point>& result)
{
	std::vector<cv::Point2f> contour;
	size_t numberOfContourPoints = __location.size();
	contour.resize(numberOfContourPoints);
	for (size_t i = 0u; i < numberOfContourPoints; ++i) {
		contour[i].x = __location[i].x;
		contour[i].y = __location[i].y;
	}

	if (!__isInit) {
		if (__location.empty()) {
			return false;
		} else {
			__target = new (std::nothrow)ImageObject;
			if (__target == NULL)
				return false;

			__target->fill(
					frame,
					__featureExtractingParams,
					contourResize(contour, __objectScalingFactor));
			__target->setContour(contour);
			__isInit = true;
			result = __location;
			return true;
		}
	}

	cv::Ptr<ImageObject> sceneImageObject = new (std::nothrow)ImageObject;
	if (sceneImageObject == NULL)
		return false;

	sceneImageObject->fill(frame, __featureExtractingParams, computeExpectedArea());

	ImageRecognizer recognizer(*sceneImageObject.get());

	const bool isTracked =
			recognizer.recognize(
					*(__target.get()),
					__recogParams,
					contour,
					__objectScalingFactor);

	if (isTracked) {
		numberOfContourPoints = contour.size();
		__location.resize(numberOfContourPoints);
		for (size_t i = 0u; i < numberOfContourPoints; ++i) {
			__location[i].x = (int)contour[i].x;
			__location[i].y = (int)contour[i].y;
		}

		result = __location;
		__target = sceneImageObject;
		__target->setContour(contour);
	} else {
		__location.clear();
		__isInit = false;
	}

	return isTracked;
}

void FeatureSubstitutionTracker::reinforcement(const std::vector<cv::Point>& location)
{
	__isInit = false;

	if (location.size() < 3) {
		__location.clear();
		return;
	}

	__location = location;
}

cv::Ptr<ObjectTracker> FeatureSubstitutionTracker::clone() const
{
	return cv::Ptr<ObjectTracker>(new (std::nothrow)FeatureSubstitutionTracker(*this));
}

std::vector<cv::Point2f> FeatureSubstitutionTracker::computeExpectedArea()
{
	std::vector<cv::Point2f> contour;
	const size_t numberOfContourPoints = __location.size();
	contour.resize(numberOfContourPoints);
	for (size_t i = 0u; i < numberOfContourPoints; ++i) {
		contour[i].x = __location[i].x;
		contour[i].y = __location[i].y;
	}

	return contourResize(contour, __sceneScalingFactor);
}

} /* Image */
} /* MediaVision */
